@@ -79,7 +79,7 @@ typedef enum {
  *                 from the pipeline and unblock all streaming threads.
  * @GST_EVENT_FLUSH_STOP: Stop a flush operation. This event resets the
  *                 running-time of the pipeline.
- * @GST_EVENT_SELECT_STREAMS: A request to select one or more streams (Since 1.10)
+ * @GST_EVENT_SELECT_STREAMS: A request to select one or more streams (Since: 1.10)
  * @GST_EVENT_STREAM_START: Event to mark the start of a new stream. Sent before any
  *                 other serialized event and only sent at the start of a new stream,
  *                 not after flushing seeks.
@@ -88,7 +88,7 @@ typedef enum {
  *                 segment events contains information for clipping buffers and
  *                 converting buffer timestamps to running-time and
  *                 stream-time.
- * @GST_EVENT_STREAM_COLLECTION: A new #GstStreamCollection is available (Since 1.10)
+ * @GST_EVENT_STREAM_COLLECTION: A new #GstStreamCollection is available (Since: 1.10)
  * @GST_EVENT_TAG: A new set of metadata tags has been found in the stream.
  * @GST_EVENT_BUFFERSIZE: Notification of buffering requirements. Currently not
  *                 used yet.
@@ -97,7 +97,7 @@ typedef enum {
  *                          rendering.
  * @GST_EVENT_STREAM_GROUP_DONE: Indicates that there is no more data for
  *                 the stream group ID in the message. Sent before EOS
- *                 in some instances and should be handled mostly the same. (Since 1.10)
+ *                 in some instances and should be handled mostly the same. (Since: 1.10)
  * @GST_EVENT_EOS: End-Of-Stream. No more data is to be expected to follow
  *                 without either a STREAM_START event, or a FLUSH_STOP and a SEGMENT
  *                 event.
@@ -121,6 +121,11 @@ typedef enum {
  * @GST_EVENT_RECONFIGURE: A request for upstream renegotiating caps and reconfiguring.
  * @GST_EVENT_TOC_SELECT: A request for a new playback position based on TOC
  *                        entry's UID.
+ * @GST_EVENT_INSTANT_RATE_CHANGE: Notify downstream that a playback rate override
+ *                                 should be applied as soon as possible. (Since: 1.18)
+ * @GST_EVENT_INSTANT_RATE_SYNC_TIME: Sent by the pipeline to notify elements that handle the
+ *                                    instant-rate-change event about the running-time when
+ *                                    the rate multiplier should be applied (or was applied). (Since: 1.18)
  * @GST_EVENT_CUSTOM_UPSTREAM: Upstream custom event
  * @GST_EVENT_CUSTOM_DOWNSTREAM: Downstream custom event that travels in the
  *                        data flow.
@@ -163,15 +168,19 @@ typedef enum {
   GST_EVENT_SEGMENT_DONE          = GST_EVENT_MAKE_TYPE (150, FLAG(DOWNSTREAM) | FLAG(SERIALIZED)),
   GST_EVENT_GAP                   = GST_EVENT_MAKE_TYPE (160, FLAG(DOWNSTREAM) | FLAG(SERIALIZED)),
 
+  /* sticky downstream non-serialized */
+  GST_EVENT_INSTANT_RATE_CHANGE   = GST_EVENT_MAKE_TYPE (180, FLAG(DOWNSTREAM) | FLAG(STICKY)),
+
   /* upstream events */
-  GST_EVENT_QOS                   = GST_EVENT_MAKE_TYPE (190, FLAG(UPSTREAM)),
-  GST_EVENT_SEEK                  = GST_EVENT_MAKE_TYPE (200, FLAG(UPSTREAM)),
-  GST_EVENT_NAVIGATION            = GST_EVENT_MAKE_TYPE (210, FLAG(UPSTREAM)),
-  GST_EVENT_LATENCY               = GST_EVENT_MAKE_TYPE (220, FLAG(UPSTREAM)),
-  GST_EVENT_STEP                  = GST_EVENT_MAKE_TYPE (230, FLAG(UPSTREAM)),
-  GST_EVENT_RECONFIGURE           = GST_EVENT_MAKE_TYPE (240, FLAG(UPSTREAM)),
-  GST_EVENT_TOC_SELECT            = GST_EVENT_MAKE_TYPE (250, FLAG(UPSTREAM)),
-  GST_EVENT_SELECT_STREAMS        = GST_EVENT_MAKE_TYPE (260, FLAG(UPSTREAM)),
+  GST_EVENT_QOS                    = GST_EVENT_MAKE_TYPE (190, FLAG(UPSTREAM)),
+  GST_EVENT_SEEK                   = GST_EVENT_MAKE_TYPE (200, FLAG(UPSTREAM)),
+  GST_EVENT_NAVIGATION             = GST_EVENT_MAKE_TYPE (210, FLAG(UPSTREAM)),
+  GST_EVENT_LATENCY                = GST_EVENT_MAKE_TYPE (220, FLAG(UPSTREAM)),
+  GST_EVENT_STEP                   = GST_EVENT_MAKE_TYPE (230, FLAG(UPSTREAM)),
+  GST_EVENT_RECONFIGURE            = GST_EVENT_MAKE_TYPE (240, FLAG(UPSTREAM)),
+  GST_EVENT_TOC_SELECT             = GST_EVENT_MAKE_TYPE (250, FLAG(UPSTREAM)),
+  GST_EVENT_SELECT_STREAMS         = GST_EVENT_MAKE_TYPE (260, FLAG(UPSTREAM)),
+  GST_EVENT_INSTANT_RATE_SYNC_TIME = GST_EVENT_MAKE_TYPE (261, FLAG(UPSTREAM)),
 
   /* custom events start here */
   GST_EVENT_CUSTOM_UPSTREAM          = GST_EVENT_MAKE_TYPE (270, FLAG(UPSTREAM)),
@@ -252,6 +261,7 @@ GST_API GType _gst_event_type;
  * Get the #GstClockTime timestamp of the event. This is the time when the event
  * was created.
  */
+/* FIXME 2.0: Remove the GstEvent::timestamp field */
 #define GST_EVENT_TIMESTAMP(event)      (GST_EVENT_CAST(event)->timestamp)
 
 /**
@@ -312,7 +322,7 @@ GST_API GType _gst_event_type;
  */
 #define         gst_event_make_writable(ev)   GST_EVENT_CAST (gst_mini_object_make_writable (GST_MINI_OBJECT_CAST (ev)))
 /**
- * gst_event_replace:
+ * gst_event_replace: (skip)
  * @old_event: (inout) (transfer full) (nullable): pointer to a
  *     pointer to a #GstEvent to be replaced.
  * @new_event: (allow-none) (transfer none): pointer to a #GstEvent that will
@@ -327,14 +337,15 @@ GST_API GType _gst_event_type;
  *
  * Returns: %TRUE if @new_event was different from @old_event
  */
+static inline gboolean gst_event_replace(GstEvent** old_event, GstEvent* new_event);
 static inline gboolean
-gst_event_replace (GstEvent **old_event, GstEvent *new_event)
+gst_event_replace(GstEvent** old_event, GstEvent* new_event)
 {
   return gst_mini_object_replace ((GstMiniObject **) old_event, (GstMiniObject *) new_event);
 }
 
 /**
- * gst_event_steal:
+ * gst_event_steal: (skip)
  * @old_event: (inout) (transfer full) (nullable): pointer to a
  *     pointer to a #GstEvent to be stolen.
  *
@@ -343,6 +354,7 @@ gst_event_replace (GstEvent **old_event, GstEvent *new_event)
  *
  * Returns: the #GstEvent that was in @old_event
  */
+static inline GstEvent* gst_event_steal(GstEvent** old_event);
 static inline GstEvent *
 gst_event_steal (GstEvent **old_event)
 {
@@ -350,7 +362,7 @@ gst_event_steal (GstEvent **old_event)
 }
 
 /**
- * gst_event_take:
+ * gst_event_take: (skip)
  * @old_event: (inout) (transfer full) (nullable): pointer to a
  *     pointer to a #GstEvent to be stolen.
  * @new_event: (allow-none) (transfer full): pointer to a #GstEvent that will
@@ -364,6 +376,7 @@ gst_event_steal (GstEvent **old_event)
  *
  * Returns: %TRUE if @new_event was different from @old_event
  */
+static inline gboolean gst_event_take(GstEvent** old_event, GstEvent* new_event);
 static inline gboolean
 gst_event_take (GstEvent **old_event, GstEvent *new_event)
 {
@@ -406,6 +419,7 @@ struct _GstEvent {
 
   /*< public >*/ /* with COW */
   GstEventType  type;
+  /* FIXME 2.0: Remove the GstEvent::timestamp field */
   guint64       timestamp;
   guint32       seqnum;
 };
@@ -423,13 +437,14 @@ GstEventTypeFlags
 
 /* refcounting */
 /**
- * gst_event_ref:
+ * gst_event_ref: (skip)
  * @event: The event to refcount
  *
  * Increase the refcount of this event.
  *
  * Returns: (transfer full): @event (for convenience when doing assignments)
  */
+static inline GstEvent* gst_event_ref(GstEvent* event);
 static inline GstEvent *
 gst_event_ref (GstEvent * event)
 {
@@ -437,26 +452,47 @@ gst_event_ref (GstEvent * event)
 }
 
 /**
- * gst_event_unref:
+ * gst_event_unref: (skip)
  * @event: (transfer full): the event to refcount
  *
  * Decrease the refcount of an event, freeing it if the refcount reaches 0.
  */
+static inline void gst_event_unref(GstEvent* event);
 static inline void
 gst_event_unref (GstEvent * event)
 {
   gst_mini_object_unref (GST_MINI_OBJECT_CAST (event));
 }
 
+/**
+ * gst_clear_event: (skip)
+ * @event_ptr: a pointer to a #GstEvent reference
+ *
+ * Clears a reference to a #GstEvent.
+ *
+ * @event_ptr must not be %NULL.
+ *
+ * If the reference is %NULL then this function does nothing. Otherwise, the
+ * reference count of the event is decreased and the pointer is set to %NULL.
+ *
+ * Since: 1.16
+ */
+static inline void
+gst_clear_event (GstEvent ** event_ptr)
+{
+  gst_clear_mini_object ((GstMiniObject **) event_ptr);
+}
+
 /* copy event */
 /**
- * gst_event_copy:
+ * gst_event_copy: (skip)
  * @event: The event to copy
  *
  * Copy the event using the event specific copy function.
  *
  * Returns: (transfer full): the new event
  */
+static inline GstEvent* gst_event_copy(const GstEvent* event);
 static inline GstEvent *
 gst_event_copy (const GstEvent * event)
 {
@@ -481,6 +517,9 @@ GstStructure *  gst_event_writable_structure    (GstEvent *event);
 GST_API
 gboolean        gst_event_has_name              (GstEvent *event, const gchar *name);
 
+GST_API
+gboolean        gst_event_has_name_id           (GstEvent *event, GQuark name);
+
 /* identifiers for events and messages */
 
 GST_API
@@ -655,6 +694,12 @@ void            gst_event_parse_seek            (GstEvent *event, gdouble *rate,
                                                  GstSeekType *start_type, gint64 *start,
                                                  GstSeekType *stop_type, gint64 *stop);
 
+GST_API
+void            gst_event_set_seek_trickmode_interval (GstEvent *event, GstClockTime interval);
+
+GST_API
+void            gst_event_parse_seek_trickmode_interval (GstEvent *event, GstClockTime *interval);
+
 /* navigation event */
 
 GST_API
@@ -698,9 +743,29 @@ GstEvent*       gst_event_new_segment_done      (GstFormat format, gint64 positi
 GST_API
 void            gst_event_parse_segment_done    (GstEvent *event, GstFormat *format, gint64 *position);
 
-#ifdef G_DEFINE_AUTOPTR_CLEANUP_FUNC
+/* instant-rate-change event */
+
+GST_API
+GstEvent *      gst_event_new_instant_rate_change   (gdouble rate_multiplier, GstSegmentFlags new_flags) G_GNUC_MALLOC;
+
+GST_API
+void            gst_event_parse_instant_rate_change (GstEvent *event,
+                                                     gdouble  *rate_multiplier, GstSegmentFlags *new_flags);
+
+/* instant-rate-change-sync-time event */
+
+GST_API
+GstEvent *      gst_event_new_instant_rate_sync_time   (gdouble      rate_multiplier,
+                                                        GstClockTime running_time,
+                                                        GstClockTime upstream_running_time) G_GNUC_MALLOC;
+
+GST_API
+void            gst_event_parse_instant_rate_sync_time (GstEvent     *event,
+                                                        gdouble      *rate_multiplier,
+                                                        GstClockTime *running_time,
+                                                        GstClockTime *upstream_running_time);
+
 G_DEFINE_AUTOPTR_CLEANUP_FUNC(GstEvent, gst_event_unref)
-#endif
 
 G_END_DECLS
 
