@@ -959,6 +959,16 @@ gst_info_printf_pointer_extension_func (const char *format, void *ptr)
       case 'B':                /* GST_SEGMENT_FORMAT */
         s = gst_debug_print_segment (ptr);
         break;
+      case 'T':                /* GST_TIMEP_FORMAT */
+        if (ptr)
+          s = g_strdup_printf ("%" GST_TIME_FORMAT,
+              GST_TIME_ARGS (*(GstClockTime *) ptr));
+        break;
+      case 'S':                /* GST_STIMEP_FORMAT */
+        if (ptr)
+          s = g_strdup_printf ("%" GST_STIME_FORMAT,
+              GST_STIME_ARGS (*(gint64 *) ptr));
+        break;
       case 'a':                /* GST_WRAPPED_PTR_FORMAT */
         s = priv_gst_string_take_and_wrap (gst_debug_print_object (ptr));
         break;
