@@ -37,7 +37,9 @@ typedef struct _GstMessage GstMessage;
  * flushing seek in the pipeline, which will undo the EOS state again.
  * @GST_MESSAGE_ERROR: an error occurred. When the application receives an error
  * message it should stop playback of the pipeline and not assume that more
- * data will be played.
+ * data will be played. It is possible to specify a redirection url to the error
+ * messages by setting a `redirect-location` field into the error message, application
+ * or high level bins might use the information as required.
  * @GST_MESSAGE_WARNING: a warning occurred.
  * @GST_MESSAGE_INFO: an info message occurred
  * @GST_MESSAGE_TAG: a tag was found.
@@ -99,26 +101,31 @@ typedef struct _GstMessage GstMessage;
  *     e.g. when using playbin in gapless playback mode, to get notified when
  *     the next title actually starts playing (which will be some time after
  *     the URI for the next title has been set).
- * @GST_MESSAGE_NEED_CONTEXT: Message indicating that an element wants a specific context (Since 1.2)
- * @GST_MESSAGE_HAVE_CONTEXT: Message indicating that an element created a context (Since 1.2)
+ * @GST_MESSAGE_NEED_CONTEXT: Message indicating that an element wants a specific context (Since: 1.2)
+ * @GST_MESSAGE_HAVE_CONTEXT: Message indicating that an element created a context (Since: 1.2)
  * @GST_MESSAGE_EXTENDED: Message is an extended message type (see below).
  *     These extended message IDs can't be used directly with mask-based API
  *     like gst_bus_poll() or gst_bus_timed_pop_filtered(), but you can still
  *     filter for GST_MESSAGE_EXTENDED and then check the result for the
- *     specific type. (Since 1.4)
+ *     specific type. (Since: 1.4)
  * @GST_MESSAGE_DEVICE_ADDED: Message indicating a #GstDevice was added to
- *     a #GstDeviceProvider (Since 1.4)
+ *     a #GstDeviceProvider (Since: 1.4)
  * @GST_MESSAGE_DEVICE_REMOVED: Message indicating a #GstDevice was removed
- *     from a #GstDeviceProvider (Since 1.4)
+ *     from a #GstDeviceProvider (Since: 1.4)
  * @GST_MESSAGE_PROPERTY_NOTIFY: Message indicating a #GObject property has
- *     changed (Since 1.10)
+ *     changed (Since: 1.10)
  * @GST_MESSAGE_STREAM_COLLECTION: Message indicating a new #GstStreamCollection
- *     is available (Since 1.10)
+ *     is available (Since: 1.10)
  * @GST_MESSAGE_STREAMS_SELECTED: Message indicating the active selection of
- *     #GstStreams has changed (Since 1.10)
+ *     #GstStreams has changed (Since: 1.10)
  * @GST_MESSAGE_REDIRECT: Message indicating to request the application to
  *     try to play the given URL(s). Useful if for example a HTTP 302/303
- *     response is received with a non-HTTP URL inside. (Since 1.10)
+ *     response is received with a non-HTTP URL inside. (Since: 1.10)
+ * @GST_MESSAGE_DEVICE_CHANGED: Message indicating a #GstDevice was changed
+ *     a #GstDeviceProvider (Since: 1.16)
+ * @GST_MESSAGE_INSTANT_RATE_REQUEST: Message sent by elements to request the
+ *     running time from the pipeline when an instant rate change should
+ *     be applied (which may be in the past when the answer arrives). (Since: 1.18)
  * @GST_MESSAGE_ANY: mask for all of the above messages.
  *
  * The different message types that are available.
@@ -171,6 +178,8 @@ typedef enum
   GST_MESSAGE_STREAM_COLLECTION = GST_MESSAGE_EXTENDED + 4,
   GST_MESSAGE_STREAMS_SELECTED  = GST_MESSAGE_EXTENDED + 5,
   GST_MESSAGE_REDIRECT          = GST_MESSAGE_EXTENDED + 6,
+  GST_MESSAGE_DEVICE_CHANGED    = GST_MESSAGE_EXTENDED + 7,
+  GST_MESSAGE_INSTANT_RATE_REQUEST = GST_MESSAGE_EXTENDED + 8,
   GST_MESSAGE_ANY               = (gint) (0xffffffff)
 } GstMessageType;
 
@@ -185,7 +194,7 @@ typedef enum
 #include <gst/gststreams.h>
 #include <gst/gststreamcollection.h>
 
-GST_EXPORT GType _gst_message_type;
+GST_API GType _gst_message_type;
 
 #define GST_TYPE_MESSAGE                         (_gst_message_type)
 #define GST_IS_MESSAGE(obj)                      (GST_IS_MINI_OBJECT_TYPE (obj, GST_TYPE_MESSAGE))
@@ -193,7 +202,7 @@ GST_EXPORT GType _gst_message_type;
 #define GST_MESSAGE(obj)                         (GST_MESSAGE_CAST(obj))
 
 /* the lock is used to handle the synchronous handling of messages,
- * the emiting thread is block until the handling thread processed
+ * the emitting thread is blocked until the handling thread processed
  * the message using this mutex/cond pair */
 #define GST_MESSAGE_GET_LOCK(message)   (&GST_MESSAGE_CAST(message)->lock)
 #define GST_MESSAGE_LOCK(message)       g_mutex_lock(GST_MESSAGE_GET_LOCK(message))
@@ -342,24 +351,25 @@ struct _GstMessage
 
 #include <gst/gstquery.h>
 
-GST_EXPORT
+GST_API
 GType           gst_message_get_type            (void);
 
-GST_EXPORT
+GST_API
 const gchar*    gst_message_type_get_name       (GstMessageType type);
 
-GST_EXPORT
+GST_API
 GQuark          gst_message_type_to_quark       (GstMessageType type);
 
 /* refcounting */
 /**
- * gst_message_ref:
+ * gst_message_ref: (skip)
  * @msg: the message to ref
  *
  * Convenience macro to increase the reference count of the message.
  *
  * Returns: @msg (for convenience when doing assignments)
  */
+static inline GstMessage * gst_message_ref (GstMessage * msg);
 static inline GstMessage *
 gst_message_ref (GstMessage * msg)
 {
@@ -367,21 +377,41 @@ gst_message_ref (GstMessage * msg)
 }
 
 /**
- * gst_message_unref:
+ * gst_message_unref: (skip)
  * @msg: the message to unref
  *
  * Convenience macro to decrease the reference count of the message, possibly
  * freeing it.
  */
+static inline void gst_message_unref (GstMessage * msg);
 static inline void
 gst_message_unref (GstMessage * msg)
 {
   gst_mini_object_unref (GST_MINI_OBJECT_CAST (msg));
 }
 
+/**
+ * gst_clear_message: (skip)
+ * @msg_ptr: a pointer to a #GstMessage reference
+ *
+ * Clears a reference to a #GstMessage.
+ *
+ * @msg_ptr must not be %NULL.
+ *
+ * If the reference is %NULL then this function does nothing. Otherwise, the
+ * reference count of the message is decreased and the pointer is set to %NULL.
+ *
+ * Since: 1.16
+ */
+static inline void
+gst_clear_message (GstMessage ** msg_ptr)
+{
+  gst_clear_mini_object ((GstMiniObject **) msg_ptr);
+}
+
 /* copy message */
 /**
- * gst_message_copy:
+ * gst_message_copy: (skip)
  * @msg: the message to copy
  *
  * Creates a copy of the message. Returns a copy of the message.
@@ -390,6 +420,7 @@ gst_message_unref (GstMessage * msg)
  *
  * MT safe
  */
+static inline GstMessage * gst_message_copy (const GstMessage * msg);
 static inline GstMessage *
 gst_message_copy (const GstMessage * msg)
 {
@@ -417,7 +448,7 @@ gst_message_copy (const GstMessage * msg)
  */
 #define         gst_message_make_writable(msg)  GST_MESSAGE_CAST (gst_mini_object_make_writable (GST_MINI_OBJECT_CAST (msg)))
 /**
- * gst_message_replace:
+ * gst_message_replace: (skip)
  * @old_message: (inout) (transfer full) (nullable): pointer to a
  *     pointer to a #GstMessage to be replaced.
  * @new_message: (allow-none) (transfer none): pointer to a #GstMessage that will
@@ -432,389 +463,426 @@ gst_message_copy (const GstMessage * msg)
  *
  * Returns: %TRUE if @new_message was different from @old_message
  */
+static inline gboolean gst_message_replace (GstMessage **old_message, GstMessage *new_message);
 static inline gboolean
 gst_message_replace (GstMessage **old_message, GstMessage *new_message)
 {
   return gst_mini_object_replace ((GstMiniObject **) old_message, (GstMiniObject *) new_message);
 }
 
+/**
+ * gst_message_take:
+ * @old_message: (inout) (transfer full): pointer to a pointer to a #GstMessage
+ *     to be replaced.
+ * @new_message: (transfer full) (allow-none): pointer to a #GstMessage that
+ *     will replace the message pointed to by @old_message.
+ *
+ * Modifies a pointer to a #GstMessage to point to a different #GstMessage. This
+ * function is similar to gst_message_replace() except that it takes ownership
+ * of @new_message.
+ *
+ * Returns: %TRUE if @new_message was different from @old_message
+ *
+ * Since: 1.16
+ */
+static inline gboolean
+gst_message_take (GstMessage **old_message, GstMessage *new_message)
+{
+  return gst_mini_object_take ((GstMiniObject **) old_message,
+      (GstMiniObject *) new_message);
+}
+
 
 /* custom messages */
 
-GST_EXPORT
+GST_API
 GstMessage *    gst_message_new_custom          (GstMessageType type,
                                                  GstObject    * src,
                                                  GstStructure * structure) G_GNUC_MALLOC;
-GST_EXPORT
+GST_API
 const GstStructure *
                 gst_message_get_structure       (GstMessage *message);
 
-GST_EXPORT
+GST_API
 GstStructure *  gst_message_writable_structure  (GstMessage *message);
 
-GST_EXPORT
+GST_API
 gboolean        gst_message_has_name            (GstMessage *message, const gchar *name);
 
 /* identifiers for events and messages */
 
-GST_EXPORT
+GST_API
 guint32         gst_message_get_seqnum          (GstMessage *message);
 
-GST_EXPORT
+GST_API
 void            gst_message_set_seqnum          (GstMessage *message, guint32 seqnum);
 
 /* EOS */
 
-GST_EXPORT
+GST_API
 GstMessage *    gst_message_new_eos             (GstObject * src) G_GNUC_MALLOC;
 
 /* ERROR */
 
-GST_EXPORT
+GST_API
 GstMessage *    gst_message_new_error           (GstObject * src, GError * error, const gchar * debug) G_GNUC_MALLOC;
 
-GST_EXPORT
+GST_API
 GstMessage *    gst_message_new_error_with_details (GstObject * src, GError * error, const gchar * debug, GstStructure * details) G_GNUC_MALLOC;
 
-GST_EXPORT
+GST_API
 void            gst_message_parse_error         (GstMessage *message, GError **gerror, gchar **debug);
 
-GST_EXPORT
+GST_API
 void            gst_message_parse_error_details (GstMessage *message, const GstStructure **structure);
 
 /* WARNING */
 
-GST_EXPORT
+GST_API
 GstMessage *    gst_message_new_warning         (GstObject * src, GError * error, const gchar * debug) G_GNUC_MALLOC;
 
-GST_EXPORT
+GST_API
 GstMessage *    gst_message_new_warning_with_details (GstObject * src, GError * error, const gchar * debug, GstStructure * details) G_GNUC_MALLOC;
 
-GST_EXPORT
+GST_API
 void            gst_message_parse_warning       (GstMessage *message, GError **gerror, gchar **debug);
 
-GST_EXPORT
+GST_API
 void            gst_message_parse_warning_details (GstMessage *message, const GstStructure **structure);
 
 /* INFO */
 
-GST_EXPORT
+GST_API
 GstMessage *    gst_message_new_info            (GstObject * src, GError * error, const gchar * debug) G_GNUC_MALLOC;
 
-GST_EXPORT
+GST_API
 GstMessage *    gst_message_new_info_with_details (GstObject * src, GError * error, const gchar * debug, GstStructure * details) G_GNUC_MALLOC;
 
-GST_EXPORT
+GST_API
 void            gst_message_parse_info          (GstMessage *message, GError **gerror, gchar **debug);
 
-GST_EXPORT
+GST_API
 void            gst_message_parse_info_details  (GstMessage *message, const GstStructure **structure);
 
 /* TAG */
 
-GST_EXPORT
+GST_API
 GstMessage *    gst_message_new_tag             (GstObject * src, GstTagList * tag_list) G_GNUC_MALLOC;
 
-GST_EXPORT
+GST_API
 void            gst_message_parse_tag           (GstMessage *message, GstTagList **tag_list);
 
 /* BUFFERING */
 
-GST_EXPORT
+GST_API
 GstMessage *    gst_message_new_buffering         (GstObject * src, gint percent) G_GNUC_MALLOC;
 
-GST_EXPORT
+GST_API
 void            gst_message_parse_buffering       (GstMessage *message, gint *percent);
 
-GST_EXPORT
+GST_API
 void            gst_message_set_buffering_stats   (GstMessage *message, GstBufferingMode mode,
                                                    gint avg_in, gint avg_out,
                                                    gint64 buffering_left);
-GST_EXPORT
+GST_API
 void            gst_message_parse_buffering_stats (GstMessage *message, GstBufferingMode *mode,
                                                    gint *avg_in, gint *avg_out,
                                                    gint64 *buffering_left);
 
 /* STATE_CHANGED */
 
-GST_EXPORT
+GST_API
 GstMessage *    gst_message_new_state_changed   (GstObject * src, GstState oldstate,
                                                  GstState newstate, GstState pending) G_GNUC_MALLOC;
-GST_EXPORT
+GST_API
 void            gst_message_parse_state_changed (GstMessage *message, GstState *oldstate,
                                                  GstState *newstate, GstState *pending);
 
 /* STATE_DIRTY */
 
-GST_EXPORT
+GST_API
 GstMessage *    gst_message_new_state_dirty     (GstObject * src) G_GNUC_MALLOC;
 
 /* STEP_DONE */
 
-GST_EXPORT
+GST_API
 GstMessage *    gst_message_new_step_done       (GstObject * src, GstFormat format, guint64 amount,
                                                  gdouble rate, gboolean flush, gboolean intermediate,
                                                  guint64 duration, gboolean eos) G_GNUC_MALLOC;
-GST_EXPORT
+GST_API
 void            gst_message_parse_step_done     (GstMessage * message, GstFormat *format, guint64 *amount,
                                                  gdouble *rate, gboolean *flush, gboolean *intermediate,
                                                  guint64 *duration, gboolean *eos);
 /* CLOCK_PROVIDE */
 
-GST_EXPORT
+GST_API
 GstMessage *    gst_message_new_clock_provide   (GstObject * src, GstClock *clock, gboolean ready) G_GNUC_MALLOC;
 
-GST_EXPORT
+GST_API
 void            gst_message_parse_clock_provide (GstMessage *message, GstClock **clock,
                                                  gboolean *ready);
 
 /* CLOCK_LOST */
 
-GST_EXPORT
+GST_API
 GstMessage *    gst_message_new_clock_lost      (GstObject * src, GstClock *clock) G_GNUC_MALLOC;
 
-GST_EXPORT
+GST_API
 void            gst_message_parse_clock_lost    (GstMessage *message, GstClock **clock);
 
 /* NEW_CLOCK */
 
-GST_EXPORT
+GST_API
 GstMessage *    gst_message_new_new_clock       (GstObject * src, GstClock *clock) G_GNUC_MALLOC;
 
-GST_EXPORT
+GST_API
 void            gst_message_parse_new_clock     (GstMessage *message, GstClock **clock);
 
 /* APPLICATION */
 
-GST_EXPORT
+GST_API
 GstMessage *    gst_message_new_application     (GstObject * src, GstStructure * structure) G_GNUC_MALLOC;
 
 /* ELEMENT */
 
-GST_EXPORT
+GST_API
 GstMessage *    gst_message_new_element         (GstObject * src, GstStructure * structure) G_GNUC_MALLOC;
 
 /* SEGMENT_START */
 
-GST_EXPORT
+GST_API
 GstMessage *    gst_message_new_segment_start   (GstObject * src, GstFormat format, gint64 position) G_GNUC_MALLOC;
 
-GST_EXPORT
+GST_API
 void            gst_message_parse_segment_start (GstMessage *message, GstFormat *format,
                                                  gint64 *position);
 
 /* SEGMENT_DONE */
 
-GST_EXPORT
+GST_API
 GstMessage *    gst_message_new_segment_done    (GstObject * src, GstFormat format, gint64 position) G_GNUC_MALLOC;
 
-GST_EXPORT
+GST_API
 void            gst_message_parse_segment_done  (GstMessage *message, GstFormat *format,
                                                  gint64 *position);
 
 /* DURATION_CHANGED */
 
-GST_EXPORT
+GST_API
 GstMessage *    gst_message_new_duration_changed (GstObject * src) G_GNUC_MALLOC;
 
 /* LATENCY */
 
-GST_EXPORT
+GST_API
 GstMessage *    gst_message_new_latency         (GstObject * src) G_GNUC_MALLOC;
 
 /* ASYNC_START */
 
-GST_EXPORT
+GST_API
 GstMessage *    gst_message_new_async_start     (GstObject * src) G_GNUC_MALLOC;
 
 /* ASYNC_DONE */
 
-GST_EXPORT
+GST_API
 GstMessage *    gst_message_new_async_done      (GstObject * src, GstClockTime running_time) G_GNUC_MALLOC;
 
-GST_EXPORT
+GST_API
 void            gst_message_parse_async_done    (GstMessage *message, GstClockTime *running_time);
 
 /* STRUCTURE CHANGE */
 
-GST_EXPORT
+GST_API
 GstMessage *    gst_message_new_structure_change   (GstObject * src, GstStructureChangeType type,
                                                     GstElement *owner, gboolean busy) G_GNUC_MALLOC;
-GST_EXPORT
+GST_API
 void            gst_message_parse_structure_change (GstMessage *message, GstStructureChangeType *type,
                                                     GstElement **owner, gboolean *busy);
 
 /* STREAM STATUS */
 
-GST_EXPORT
+GST_API
 GstMessage *    gst_message_new_stream_status        (GstObject * src, GstStreamStatusType type,
                                                       GstElement *owner) G_GNUC_MALLOC;
-GST_EXPORT
+GST_API
 void            gst_message_parse_stream_status      (GstMessage *message, GstStreamStatusType *type,
                                                       GstElement **owner);
-GST_EXPORT
+GST_API
 void            gst_message_set_stream_status_object (GstMessage *message, const GValue *object);
 
-GST_EXPORT
+GST_API
 const GValue *  gst_message_get_stream_status_object (GstMessage *message);
 
 /* REQUEST_STATE */
 
-GST_EXPORT
+GST_API
 GstMessage *    gst_message_new_request_state   (GstObject * src, GstState state) G_GNUC_MALLOC;
 
-GST_EXPORT
+GST_API
 void            gst_message_parse_request_state (GstMessage * message, GstState *state);
 
 /* STEP_START */
 
-GST_EXPORT
+GST_API
 GstMessage *    gst_message_new_step_start      (GstObject * src, gboolean active, GstFormat format,
                                                  guint64 amount, gdouble rate, gboolean flush,
                                                  gboolean intermediate) G_GNUC_MALLOC;
-GST_EXPORT
+GST_API
 void            gst_message_parse_step_start    (GstMessage * message, gboolean *active, GstFormat *format,
                                                  guint64 *amount, gdouble *rate, gboolean *flush,
                                                  gboolean *intermediate);
 
 /* QOS */
 
-GST_EXPORT
+GST_API
 GstMessage *    gst_message_new_qos             (GstObject * src, gboolean live, guint64 running_time,
                                                  guint64 stream_time, guint64 timestamp, guint64 duration) G_GNUC_MALLOC;
-GST_EXPORT
+GST_API
 void            gst_message_set_qos_values      (GstMessage * message, gint64 jitter, gdouble proportion,
                                                  gint quality);
-GST_EXPORT
+GST_API
 void            gst_message_set_qos_stats       (GstMessage * message, GstFormat format, guint64 processed,
                                                  guint64 dropped);
-GST_EXPORT
+GST_API
 void            gst_message_parse_qos           (GstMessage * message, gboolean * live, guint64 * running_time,
                                                  guint64 * stream_time, guint64 * timestamp, guint64 * duration);
-GST_EXPORT
+GST_API
 void            gst_message_parse_qos_values    (GstMessage * message, gint64 * jitter, gdouble * proportion,
                                                  gint * quality);
-GST_EXPORT
+GST_API
 void            gst_message_parse_qos_stats     (GstMessage * message, GstFormat * format, guint64 * processed,
                                                  guint64 * dropped);
 /* PROGRESS */
 
-GST_EXPORT
+GST_API
 GstMessage *    gst_message_new_progress           (GstObject * src, GstProgressType type, const gchar *code,
                                                     const gchar *text) G_GNUC_MALLOC;
-GST_EXPORT
+GST_API
 void            gst_message_parse_progress         (GstMessage * message, GstProgressType * type, gchar ** code,
                                                     gchar ** text);
 
 /* TOC */
 
-GST_EXPORT
+GST_API
 GstMessage *    gst_message_new_toc             (GstObject *src, GstToc *toc, gboolean updated);
 
-GST_EXPORT
+GST_API
 void            gst_message_parse_toc           (GstMessage *message, GstToc **toc, gboolean *updated);
 
 /* RESET_TIME */
 
-GST_EXPORT
+GST_API
 GstMessage *    gst_message_new_reset_time      (GstObject * src, GstClockTime running_time) G_GNUC_MALLOC;
 
-GST_EXPORT
+GST_API
 void            gst_message_parse_reset_time    (GstMessage *message, GstClockTime *running_time);
 
 /* STREAM_START */
 
-GST_EXPORT
+GST_API
 GstMessage *    gst_message_new_stream_start    (GstObject * src) G_GNUC_MALLOC;
 
-GST_EXPORT
+GST_API
 void            gst_message_set_group_id        (GstMessage *message, guint group_id);
 
-GST_EXPORT
+GST_API
 gboolean        gst_message_parse_group_id      (GstMessage *message, guint *group_id);
 
 /* NEED_CONTEXT */
 
-GST_EXPORT
+GST_API
 GstMessage *    gst_message_new_need_context    (GstObject * src, const gchar * context_type) G_GNUC_MALLOC;
 
-GST_EXPORT
+GST_API
 gboolean        gst_message_parse_context_type  (GstMessage * message, const gchar ** context_type);
 
 /* HAVE_CONTEXT */
 
-GST_EXPORT
+GST_API
 GstMessage *    gst_message_new_have_context    (GstObject * src, GstContext *context) G_GNUC_MALLOC;
 
-GST_EXPORT
+GST_API
 void            gst_message_parse_have_context  (GstMessage *message, GstContext **context);
 
 /* DEVICE_ADDED */
 
-GST_EXPORT
+GST_API
 GstMessage *    gst_message_new_device_added    (GstObject * src, GstDevice * device) G_GNUC_MALLOC;
 
-GST_EXPORT
+GST_API
 void            gst_message_parse_device_added  (GstMessage * message, GstDevice ** device);
 
 /* DEVICE_REMOVED */
 
-GST_EXPORT
+GST_API
 GstMessage *    gst_message_new_device_removed    (GstObject * src, GstDevice * device) G_GNUC_MALLOC;
 
-GST_EXPORT
+GST_API
 void            gst_message_parse_device_removed  (GstMessage * message, GstDevice ** device);
 
+/* DEVICE_CHANGED */
+
+GST_API
+GstMessage *    gst_message_new_device_changed    (GstObject * src, GstDevice * device, GstDevice *changed_device);
+
+GST_API
+void            gst_message_parse_device_changed  (GstMessage * message, GstDevice ** device, GstDevice ** changed_device);
+
+
 /* PROPERTY_NOTIFY */
 
-GST_EXPORT
+GST_API
 GstMessage *    gst_message_new_property_notify   (GstObject * src, const gchar * property_name, GValue * val) G_GNUC_MALLOC;
 
-GST_EXPORT
+GST_API
 void            gst_message_parse_property_notify (GstMessage * message, GstObject ** object, const gchar ** property_name, const GValue ** property_value);
 
 /* STREAM_COLLECTION */
 
-GST_EXPORT
+GST_API
 GstMessage *    gst_message_new_stream_collection   (GstObject * src, GstStreamCollection * collection) G_GNUC_MALLOC;
 
-GST_EXPORT
+GST_API
 void            gst_message_parse_stream_collection (GstMessage *message, GstStreamCollection **collection);
 
 /* STREAMS_SELECTED */
 
-GST_EXPORT
+GST_API
 GstMessage *    gst_message_new_streams_selected (GstObject *src, GstStreamCollection *collection);
 
-GST_EXPORT
+GST_API
 void            gst_message_streams_selected_add (GstMessage *message, GstStream *stream);
 
-GST_EXPORT
+GST_API
 void            gst_message_parse_streams_selected (GstMessage * message, GstStreamCollection **collection);
 
-GST_EXPORT
+GST_API
 guint           gst_message_streams_selected_get_size (GstMessage * message);
 
-GST_EXPORT
+GST_API
 GstStream      *gst_message_streams_selected_get_stream (GstMessage *message, guint idx);
 
 /* REDIRECT */
 
-GST_EXPORT
+GST_API
 GstMessage *    gst_message_new_redirect             (GstObject * src, const gchar * location, GstTagList * tag_list, const GstStructure * entry_struct) G_GNUC_MALLOC;
 
-GST_EXPORT
+GST_API
 void            gst_message_add_redirect_entry       (GstMessage * message, const gchar * location, GstTagList * tag_list, const GstStructure * entry_struct);
 
-GST_EXPORT
+GST_API
 void            gst_message_parse_redirect_entry     (GstMessage * message, gsize entry_index, const gchar ** location, GstTagList ** tag_list, const GstStructure ** entry_struct);
 
-GST_EXPORT
+GST_API
 gsize           gst_message_get_num_redirect_entries (GstMessage * message);
 
-#ifdef G_DEFINE_AUTOPTR_CLEANUP_FUNC
+/* INSTANT_RATE_REQUEST */
+
+GST_API
+GstMessage *    gst_message_new_instant_rate_request   (GstObject * src, gdouble rate_multiplier) G_GNUC_MALLOC;
+GST_API
+void            gst_message_parse_instant_rate_request (GstMessage * message, gdouble * rate_multiplier);
+
 G_DEFINE_AUTOPTR_CLEANUP_FUNC(GstMessage, gst_message_unref)
-#endif
 
 G_END_DECLS
 
