@@ -49,65 +49,16 @@ G_BEGIN_DECLS
  * In the table below this is shown for the cases that a tag exists in the list
  * (A) or does not exists (!A) and combinations thereof.
  *
- * <table frame="all" colsep="1" rowsep="1">
- *   <title>merge mode</title>
- *   <tgroup cols='5' align='left'>
- *     <thead>
- *       <row>
- *         <entry>merge mode</entry>
- *         <entry>A + B</entry>
- *         <entry>A + !B</entry>
- *         <entry>!A + B</entry>
- *         <entry>!A + !B</entry>
- *       </row>
- *     </thead>
- *     <tbody>
- *       <row>
- *         <entry>REPLACE_ALL</entry>
- *         <entry>B</entry>
- *         <entry>-</entry>
- *         <entry>B</entry>
- *         <entry>-</entry>
- *       </row>
- *       <row>
- *         <entry>REPLACE</entry>
- *         <entry>B</entry>
- *         <entry>A</entry>
- *         <entry>B</entry>
- *         <entry>-</entry>
- *       </row>
- *       <row>
- *         <entry>APPEND</entry>
- *         <entry>A, B</entry>
- *         <entry>A</entry>
- *         <entry>B</entry>
- *         <entry>-</entry>
- *       </row>
- *       <row>
- *         <entry>PREPEND</entry>
- *         <entry>B, A</entry>
- *         <entry>A</entry>
- *         <entry>B</entry>
- *         <entry>-</entry>
- *       </row>
- *       <row>
- *         <entry>KEEP</entry>
- *         <entry>A</entry>
- *         <entry>A</entry>
- *         <entry>B</entry>
- *         <entry>-</entry>
- *       </row>
- *       <row>
- *         <entry>KEEP_ALL</entry>
- *         <entry>A</entry>
- *         <entry>A</entry>
- *         <entry>-</entry>
- *         <entry>-</entry>
- *       </row>
- *     </tbody>
- *   </tgroup>
- * </table>
+ * | merge mode  | A + B | A + !B | !A + B | !A + !B |
+ * | ----------- | ----- | ------ | ------ | ------- |
+ * | REPLACE_ALL | B     | ø      | B      | ø       |
+ * | REPLACE     | B     | A      | B      | ø       |
+ * | APPEND      | A, B  | A      | B      | ø       |
+ * | PREPEND     | B, A  | A      | B      | ø       |
+ * | KEEP        | A     | A      | B      | ø       |
+ * | KEEP_ALL    | A     | A      | ø      | ø       |
  */
+
 typedef enum {
   GST_TAG_MERGE_UNDEFINED,
   GST_TAG_MERGE_REPLACE_ALL,
@@ -450,7 +401,7 @@ gboolean     gst_tag_list_get_sample_index  (const GstTagList * list,
 
 /* refcounting */
 /**
- * gst_tag_list_ref:
+ * gst_tag_list_ref: (skip)
  * @taglist: the #GstTagList to reference
  *
  * Add a reference to a #GstTagList mini object.
@@ -463,6 +414,7 @@ gboolean     gst_tag_list_get_sample_index  (const GstTagList * list,
  *
  * Returns: the same #GstTagList mini object.
  */
+static inline GstTagList* gst_tag_list_ref(GstTagList* taglist);
 static inline GstTagList *
 gst_tag_list_ref (GstTagList * taglist)
 {
@@ -470,11 +422,12 @@ gst_tag_list_ref (GstTagList * taglist)
 }
 
 /**
- * gst_tag_list_unref:
+ * gst_tag_list_unref: (skip)
  * @taglist: a #GstTagList.
  *
  * Unref a #GstTagList, and and free all its memory when the refcount reaches 0.
  */
+static inline void gst_tag_list_unref(GstTagList* taglist);
 static inline void
 gst_tag_list_unref (GstTagList * taglist)
 {
@@ -501,7 +454,7 @@ gst_clear_tag_list (GstTagList ** taglist_ptr)
 }
 
 /**
- * gst_tag_list_copy:
+ * gst_tag_list_copy: (skip)
  * @taglist: a #GstTagList.
  *
  * Creates a new #GstTagList as a copy of the old @taglist. The new taglist
@@ -516,6 +469,7 @@ gst_clear_tag_list (GstTagList ** taglist_ptr)
  *
  * Returns: the new #GstTagList
  */
+static inline GstTagList* gst_tag_list_copy(const GstTagList* taglist);
 static inline GstTagList *
 gst_tag_list_copy (const GstTagList * taglist)
 {
@@ -938,7 +892,7 @@ gst_tag_list_take (GstTagList **old_taglist, GstTagList *new_taglist)
  * ISO-639-2 or ISO-639-1 code for the language the content is in (string)
  *
  * There is utility API in libgsttag in gst-plugins-base to obtain a translated
- * language name from the language code: gst_tag_get_language_name()
+ * language name from the language code: `gst_tag_get_language_name()`
  */
 #define GST_TAG_LANGUAGE_CODE          "language-code"
 /**
@@ -955,7 +909,7 @@ gst_tag_list_take (GstTagList **old_taglist, GstTagList *new_taglist)
  * GST_TAG_IMAGE:
  *
  * image (sample) (sample taglist should specify the content type and preferably
- * also set "image-type" field as #GstTagImageType)
+ * also set "image-type" field as `GstTagImageType`)
  */
 #define GST_TAG_IMAGE                  "image"
 /**
@@ -1202,7 +1156,7 @@ gst_tag_list_take (GstTagList **old_taglist, GstTagList *new_taglist)
 /**
  * GST_TAG_MIDI_BASE_NOTE:
  *
- * <ulink url="http://en.wikipedia.org/wiki/Note#Note_designation_in_accordance_with_octave_name">Midi note number</ulink>
+ * [Midi note number](http://en.wikipedia.org/wiki/Note#Note_designation_in_accordance_with_octave_name)
  * of the audio track. This is useful for sample instruments and in particular
  * for multi-samples.
  *
@@ -1227,9 +1181,7 @@ gst_tag_list_take (GstTagList **old_taglist, GstTagList *new_taglist)
  */
 #define GST_TAG_PRIVATE_DATA                         "private-data"
 
-#ifdef G_DEFINE_AUTOPTR_CLEANUP_FUNC
 G_DEFINE_AUTOPTR_CLEANUP_FUNC(GstTagList, gst_tag_list_unref)
-#endif
 
 G_END_DECLS
 
