@@ -21,6 +21,7 @@
  */
 /**
  * SECTION:gstdirectcontrolbinding
+ * @title: GstDirectControlBinding
  * @short_description: direct attachment for control sources
  *
  * A value mapping object that attaches control sources to gobject properties. It
@@ -30,6 +31,9 @@
  * will be clipped. An absolute control binding will not do any value
  * transformations.
  */
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include <glib-object.h>
 #include <gst/gst.h>
@@ -423,7 +427,7 @@ gst_direct_control_binding_get_value_array (GstControlBinding * _self,
     gpointer values_)
 {
   GstDirectControlBinding *self = GST_DIRECT_CONTROL_BINDING (_self);
-  gint i;
+  guint i;
   gdouble *src_val;
   gboolean res = FALSE;
   GstDirectControlBindingConvertValue convert;
@@ -466,7 +470,7 @@ gst_direct_control_binding_get_g_value_array (GstControlBinding * _self,
     GValue * values)
 {
   GstDirectControlBinding *self = GST_DIRECT_CONTROL_BINDING (_self);
-  gint i;
+  guint i;
   gdouble *src_val;
   gboolean res = FALSE;
   GType type;
