@@ -17,6 +17,13 @@
  * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
  * Boston, MA 02110-1301, USA.
  */
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
+#ifndef GST_REMOVE_DEPRECATED
+#undef GST_DISABLE_DEPRECATED
+#endif
 
 #include <gst/check/gstcheck.h>
 
@@ -75,6 +82,39 @@ GST_START_TEST (test_uri_get_location)
 
 GST_END_TEST;
 
+#ifndef GST_REMOVE_DEPRECATED
+GST_START_TEST (test_gst_uri_construct)
+{
+  gchar *l = NULL;
+
+  /* URI with no protocol or empty protocol should return empty string */
+  ASSERT_CRITICAL (l = gst_uri_construct (NULL, "/path/to/file"));
+  fail_unless (l == NULL);
+  ASSERT_CRITICAL (l = gst_uri_construct ("", "/path/to/file"));
+  fail_unless (l == NULL);
+
+  /* URI with no location should return empty string */
+  ASSERT_CRITICAL (l = gst_uri_construct ("protocol", NULL));
+  fail_unless (l == NULL);
+
+  /* check the protocol for validity */
+  l = gst_uri_construct ("protocol1234567890+-.", "somefile");
+  fail_unless (l != NULL);
+  fail_unless_equals_string (l, "protocol1234567890+-.://somefile");
+  g_free (l);
+
+  /* check the location for correct handling */
+  l = gst_uri_construct ("aprotocol",
+      "/path+ to/some/file%d?akey=aval&key2=val2");
+  fail_unless (l != NULL);
+  fail_unless_equals_string (l,
+      "aprotocol:///path%2B%20to/some/file%25d?akey=aval&key2=val2");
+  g_free (l);
+}
+
+GST_END_TEST;
+#endif
+
 #ifdef G_OS_WIN32
 
 GST_START_TEST (test_win32_uri)
@@ -374,7 +414,11 @@ static const struct URITest url_presenting_tests[] = {
   {.uri = {"scheme", "user:pass", "host", 1234, "/path/to/dir",
           {{"query", NULL}, {"key", "value"}}, "fragment"},
       .str =
+#if GLIB_CHECK_VERSION(2, 59, 0)
+      "scheme://user:pass@host:1234/path/to/dir?key=value&query#fragment"},
+#else
       "scheme://user:pass@host:1234/path/to/dir?query&key=value#fragment"},
+#endif
 
   /* IPv6 literal should render in square brackets */
   {.uri = {"scheme", "user:pass", "12:34:56:78:9a:bc:de:f0", 1234,
@@ -800,9 +844,23 @@ GST_START_TEST (test_url_constructors)
   g_free (tmp_str);
   gst_uri_unref (url2);
 
+  url2 = gst_uri_from_string_with_base (url1, "http://foobar.com/bla");
+  fail_unless_equals_string (gst_uri_get_scheme (url2), "http");
+  fail_unless_equals_string (gst_uri_get_host (url2), "foobar.com");
+  fail_unless (gst_uri_get_port (url2) == 0);
+  tmp_str = gst_uri_get_path (url2);
+  fail_unless_equals_string (tmp_str, "/bla");
+  g_free (tmp_str);
+  fail_unless (gst_uri_get_query_table (url2) == NULL);
+  fail_unless (gst_uri_get_fragment (url2) == NULL);
+  tmp_str = gst_uri_to_string (url2);
+  fail_unless_equals_string (tmp_str, "http://foobar.com/bla");
+  g_free (tmp_str);
+  gst_uri_unref (url2);
+
   url2 = gst_uri_copy (url1);
   fail_unless (gst_uri_equal (url1, url2));
-  gst_uri_set_query_value (url2, "key", "value");
+  fail_unless (gst_uri_set_query_value (url2, "key", "value"));
   fail_unless (!gst_uri_equal (url1, url2));
   gst_uri_unref (url2);
 
@@ -923,14 +981,24 @@ GST_START_TEST (test_url_get_set)
 
   fail_unless (gst_uri_set_query_value (url, "key", "value"));
   tmp_str = gst_uri_to_string (url);
+#if GLIB_CHECK_VERSION(2, 59, 0)
+  fail_unless_equals_string (tmp_str,
+      "//example.com/path/to/file/there/segment?key=value&query#fragment");
+#else
   fail_unless_equals_string (tmp_str,
       "//example.com/path/to/file/there/segment?query&key=value#fragment");
+#endif
   g_free (tmp_str);
 
   fail_unless (gst_uri_set_query_value (url, "key", NULL));
   tmp_str = gst_uri_to_string (url);
+#if GLIB_CHECK_VERSION(2, 59, 0)
+  fail_unless_equals_string (tmp_str,
+      "//example.com/path/to/file/there/segment?key&query#fragment");
+#else
   fail_unless_equals_string (tmp_str,
       "//example.com/path/to/file/there/segment?query&key#fragment");
+#endif
   g_free (tmp_str);
 
   fail_unless (!gst_uri_set_query_value (NULL, "key", "value"));
@@ -966,6 +1034,80 @@ GST_START_TEST (test_url_get_set)
 
 GST_END_TEST;
 
+GST_START_TEST (test_url_get_media_fragment_table)
+{
+  GstUri *url;
+  gchar *val;
+  GHashTable *table;
+
+  /* Examples at https://www.w3.org/TR/media-frags/#processing-media-fragment-uri */
+
+  /* TEST "t=1" */
+  url = gst_uri_from_string ("http://foo/var/file#t=1");
+  table = gst_uri_get_media_fragment_table (url);
+  fail_unless (table);
+  fail_unless (g_hash_table_size (table) == 1);
+  fail_unless (g_hash_table_lookup_extended (table, "t", NULL,
+          (gpointer) & val));
+  fail_unless_equals_string ("1", val);
+  g_hash_table_unref (table);
+  gst_uri_unref (url);
+
+  /* NOTE: Media Fragments URI 1.0 (W3C) is saying that
+   * "Multiple occurrences of the same dimension: only the last valid occurrence
+   *  of a dimension (e.g. t=10 in #t=2&t=10) is interpreted and all previous
+   *  occurrences (valid or invalid) SHOULD be ignored by the user agent"
+   */
+  /* TEST "t=1&t=2" */
+  url = gst_uri_from_string ("http://foo/var/file#t=1&t=2");
+  table = gst_uri_get_media_fragment_table (url);
+  fail_unless (table);
+  fail_unless (g_hash_table_size (table) == 1);
+  fail_unless (g_hash_table_lookup_extended (table, "t", NULL,
+          (gpointer) & val));
+  fail_unless_equals_string ("2", val);
+  g_hash_table_unref (table);
+  gst_uri_unref (url);
+
+  /* TEST "a=b=c" */
+  url = gst_uri_from_string ("http://foo/var/file#a=b=c");
+  table = gst_uri_get_media_fragment_table (url);
+  fail_unless (table);
+  fail_unless (g_hash_table_size (table) == 1);
+  fail_unless (g_hash_table_lookup_extended (table, "a", NULL,
+          (gpointer) & val));
+  fail_unless_equals_string ("b=c", val);
+  g_hash_table_unref (table);
+  gst_uri_unref (url);
+
+  /* TEST "a&b=c" */
+  url = gst_uri_from_string ("http://foo/var/file#a&b=c");
+  table = gst_uri_get_media_fragment_table (url);
+  fail_unless (table);
+  fail_unless (g_hash_table_size (table) == 2);
+  fail_unless (g_hash_table_lookup_extended (table, "a", NULL,
+          (gpointer) & val));
+  fail_unless (val == NULL);
+  fail_unless (g_hash_table_lookup_extended (table, "b", NULL,
+          (gpointer) & val));
+  fail_unless_equals_string ("c", val);
+  g_hash_table_unref (table);
+  gst_uri_unref (url);
+
+  /* TEST "%74=%6ept%3A%310" */
+  url = gst_uri_from_string ("http://foo/var/file#%74=%6ept%3A%310");
+  table = gst_uri_get_media_fragment_table (url);
+  fail_unless (table);
+  fail_unless (g_hash_table_size (table) == 1);
+  fail_unless (g_hash_table_lookup_extended (table, "t", NULL,
+          (gpointer) & val));
+  fail_unless_equals_string ("npt:10", val);
+  g_hash_table_unref (table);
+  gst_uri_unref (url);
+}
+
+GST_END_TEST;
+
 static Suite *
 gst_uri_suite (void)
 {
@@ -977,6 +1119,9 @@ gst_uri_suite (void)
   suite_add_tcase (s, tc_chain);
   tcase_add_test (tc_chain, test_protocol_case);
   tcase_add_test (tc_chain, test_uri_get_location);
+#ifndef GST_REMOVE_DEPRECATED
+  tcase_add_test (tc_chain, test_gst_uri_construct);
+#endif
   tcase_add_test (tc_chain, test_uri_misc);
   tcase_add_test (tc_chain, test_element_make_from_uri);
 #ifdef G_OS_WIN32
@@ -989,6 +1134,7 @@ gst_uri_suite (void)
   tcase_add_test (tc_chain, test_url_equality);
   tcase_add_test (tc_chain, test_url_constructors);
   tcase_add_test (tc_chain, test_url_get_set);
+  tcase_add_test (tc_chain, test_url_get_media_fragment_table);
 
   return s;
 }
