@@ -232,7 +232,7 @@ print_factory_details_info (GstElementFactory * factory)
 {
   gchar **keys, **k;
   GstRank rank;
-  char s[20];
+  char s[40];
 
   rank = gst_plugin_feature_get_rank (GST_PLUGIN_FEATURE (factory));
   n_print ("%sFactory Details:%s\n", HEADING_COLOR, RESET_COLOR);
@@ -347,7 +347,8 @@ flags_to_string (GFlagsValue * vals, guint flags)
   G_PARAM_LAX_VALIDATION |  G_PARAM_STATIC_STRINGS | \
   G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_DEPRECATED | \
   GST_PARAM_CONTROLLABLE | GST_PARAM_MUTABLE_PLAYING | \
-  GST_PARAM_MUTABLE_PAUSED | GST_PARAM_MUTABLE_READY)
+  GST_PARAM_MUTABLE_PAUSED | GST_PARAM_MUTABLE_READY | \
+  GST_PARAM_CONDITIONALLY_AVAILABLE)
 
 static int
 sort_gparamspecs (GParamSpec ** a, GParamSpec ** b)
@@ -422,6 +423,11 @@ print_object_properties_info (GObject * obj, GObjectClass * obj_class,
           RESET_COLOR);
       first_flag = FALSE;
     }
+    if (param->flags & GST_PARAM_CONDITIONALLY_AVAILABLE) {
+      g_print (", %s%s%s", PROP_ATTR_VALUE_COLOR, _("conditionally available"),
+          RESET_COLOR);
+      first_flag = FALSE;
+    }
     if (param->flags & GST_PARAM_MUTABLE_PLAYING) {
       g_print (", %s%s%s", PROP_ATTR_VALUE_COLOR,
           _("changeable in NULL, READY, PAUSED or PLAYING state"), RESET_COLOR);
@@ -1617,7 +1623,7 @@ print_typefind_info (GstPluginFeature * feature, gboolean print_names)
   GstPlugin *plugin;
   GstCaps *caps;
   GstRank rank;
-  char s[20];
+  char s[40];
   const gchar *const *extensions;
 
   factory = GST_TYPE_FIND_FACTORY (gst_plugin_feature_load (feature));
@@ -1947,6 +1953,7 @@ main (int argc, char *argv[])
   gboolean print_aii = FALSE;
   gboolean uri_handlers = FALSE;
   gboolean check_exists = FALSE;
+  gboolean color_always = FALSE;
   gchar *min_version = NULL;
   guint minver_maj = GST_VERSION_MAJOR;
   guint minver_min = GST_VERSION_MINOR;
@@ -1986,6 +1993,9 @@ main (int argc, char *argv[])
           ("Disable colors in output. You can also achieve the same by setting"
               "'GST_INSPECT_NO_COLORS' environment variable to any value."),
         NULL},
+    {"color", 'C', 0, G_OPTION_ARG_NONE, &color_always,
+          N_("Color output, even when not sending to a tty."),
+        NULL},
     GST_TOOLS_GOPTION_VERSION,
     {NULL}
   };
@@ -2080,7 +2090,7 @@ main (int argc, char *argv[])
     if (redirect_stdout ())
       loop = g_main_loop_new (NULL, FALSE);
   } else {
-    colored_output = FALSE;
+    colored_output = (color_always) ? TRUE : FALSE;
   }
 #elif defined(G_OS_WIN32)
   {
